/*
 * drivers/serial/8250_voyagergx.c
 *
 * VoyagerGX (Silicon Motion SM501) UART support.
 *
 *  Copyright (C) 2004  Paul Mundt
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 */
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/serial_core.h>
#include <linux/serial.h>
#include <asm/io.h>
#include <asm/mach/rts7751r2d.h>
#include <asm/mach/voyagergx_reg.h>

#include "8250.h"

static struct uart_port voyagergx_uart = {
	.membase	= (void *)VOYAGER_UART_BASE,
	.mapbase	= VOYAGER_UART_BASE,
	.iotype		= SERIAL_IO_MEM,
	.irq		= VOYAGER_UART0_IRQ,
	.flags		= UPF_BOOT_AUTOCONF,
	.fifosize	= 64,
	.regshift	= 2,
	.uartclk	= (9600 * 16),
};

static int __init voyagergx_serial_init(void)
{
	unsigned long val;

	/*
	 * GPIO Control
	 */
	val = inl(GPIO_MUX_HIGH);
	val |= 0x00001fe0;
	outl(val, GPIO_MUX_HIGH);

	/*
	 * Power Mode Gate
	 */
	val = inl(POWER_MODE0_GATE);
	val |= (POWER_MODE0_GATE_U0 | POWER_MODE0_GATE_U1);
	outl(val, POWER_MODE0_GATE);

	val = inl(POWER_MODE1_GATE);
	val |= (POWER_MODE1_GATE_U0 | POWER_MODE1_GATE_U1);
	outl(val, POWER_MODE1_GATE);

	return serial8250_register_port(&voyagergx_uart);
}

static void __exit voyagergx_serial_exit(void)
{
	serial8250_unregister_port(voyagergx_uart.line);
}

module_init(voyagergx_serial_init);
module_exit(voyagergx_serial_exit);

MODULE_AUTHOR("Paul Mundt");
MODULE_DESCRIPTION("VoyagerGX (Silicon Motion SM501) 8250-compatible UART");
MODULE_LICENSE("GPL");

